@@ -47,6 +47,8 @@ pub mod error;
 type Result<T> = std::result::Result<T, error::Error>;
 
 
+pub use ssh::{CertType, Certificate, PublicKey, PrivateKey};
+
 /// Functions or structs for dealing with SSH Certificates.
 /// Parsing, and creating certs happens here.
 pub mod ssh;
@@ -65,6 +67,4 @@ pub mod yubikey;
 /// certificates and CSRs. Is enabled whenever yubikey_support is enabled
 /// because some functionality is currently shared.
 #[cfg(any(feature = "yubikey-support", feature = "x509-support"))]
-pub mod x509;
-
-pub use ssh::{CertType, Certificate, PublicKey, PrivateKey, CriticalOptions, Extensions};
\ No newline at end of file
+pub mod x509;
\ No newline at end of file
