@@ -1,7 +1,6 @@
 use ring::{rand, signature};
 
-use sshcerts::ssh::{Certificate, CertType, CriticalOptions, Extensions, PrivateKey, PublicKey};
-use sshcerts::ssh::{SigningFunction, create_signer};
+use sshcerts::ssh::{Certificate, CertType, PrivateKey, PublicKey};
 
 // Constants available for multiple tests
 const ECDSA256_CA_PRIVATE_KEY: &str = concat!(
@@ -50,6 +49,7 @@ fn test_ecdsa384_signer(buf: &[u8]) -> Option<Vec<u8>> {
     Some(signature)
 }
 
+
 #[test]
 fn create_sign_parse_verify_ecdsa256_static_function() {
     let ssh_pubkey = PublicKey::from_string("ecdsa-sha2-nistp256 AAAAE2VjZHNhLXNoYTItbmlzdHAyNTYAAAAIbmlzdHAyNTYAAABBBOhHAGJtT9s6zPW4OdQMzGbXEyj0ntkESrE1IZBgaCUSh9fWK1gRz+UJOcCB1JTC/kF2EPlwkX6XEpQToZl51oo= obelisk@exclave.lan");
@@ -61,16 +61,18 @@ fn create_sign_parse_verify_ecdsa256_static_function() {
     let ssh_pubkey = ssh_pubkey.unwrap();
     let ca_pubkey = ca_pubkey.unwrap();
 
-    let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &ca_pubkey).unwrap()
+    let user_cert_partial = Certificate::builder(&ssh_pubkey, CertType::User, &ca_pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
         .key_id("key_id")
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(test_ecdsa256_signer);
+        .set_extensions(Certificate::standard_extensions());
 
+    let signature = test_ecdsa256_signer(&user_cert_partial.tbs_certificate());
+    assert!(signature.is_some());
+
+    let user_cert = user_cert_partial.add_signature(&signature.unwrap());
     assert!(user_cert.is_ok());
 
     let user_cert = user_cert.unwrap();
@@ -93,16 +95,18 @@ fn create_sign_parse_verify_ecdsa384_static_function() {
     let ssh_pubkey = ssh_pubkey.unwrap();
     let ca_pubkey = ca_pubkey.unwrap();
 
-    let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &ca_pubkey).unwrap()
+    let user_cert_partial = Certificate::builder(&ssh_pubkey, CertType::User, &ca_pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
         .key_id("key_id")
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(test_ecdsa384_signer);
+        .set_extensions(Certificate::standard_extensions());
 
+    let signature = test_ecdsa384_signer(&user_cert_partial.tbs_certificate());
+    assert!(signature.is_some());
+
+    let user_cert = user_cert_partial.add_signature(&signature.unwrap());
     assert!(user_cert.is_ok());
 
     let user_cert = user_cert.unwrap();
@@ -138,7 +142,6 @@ fn create_sign_parse_verify_ed25519ca_into_impl() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -146,9 +149,8 @@ fn create_sign_parse_verify_ed25519ca_into_impl() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
@@ -196,9 +198,8 @@ fn create_sign_parse_verify_ed25519ca_create_signer() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(create_signer(privkey));
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
@@ -241,7 +242,6 @@ fn create_sign_parse_verify_ecdsa256_into_impl() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -249,9 +249,8 @@ fn create_sign_parse_verify_ecdsa256_into_impl() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
 
     assert!(user_cert.is_ok());
@@ -301,9 +300,8 @@ fn create_sign_parse_verify_ecdsa256_create_signer() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(create_signer(privkey));
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
@@ -347,7 +345,6 @@ fn create_sign_parse_verify_ecdsa384_into_impl() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -355,9 +352,8 @@ fn create_sign_parse_verify_ecdsa384_into_impl() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
@@ -407,9 +403,8 @@ fn create_sign_parse_verify_ecdsa384_create_signer() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(create_signer(privkey));
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
@@ -493,7 +488,6 @@ fn create_sign_parse_verify_rsa4096_impl_into() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let priv_pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &priv_pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -501,9 +495,8 @@ fn create_sign_parse_verify_rsa4096_impl_into() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     match &user_cert {
         Ok(_) => (),
@@ -582,7 +575,6 @@ fn create_sign_parse_verify_rsa3072_impl_into() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let priv_pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &priv_pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -590,9 +582,8 @@ fn create_sign_parse_verify_rsa3072_impl_into() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     match &user_cert {
         Ok(_) => (),
@@ -660,7 +651,6 @@ fn create_sign_parse_verify_rsa2048_impl_into() {
 
     let ssh_pubkey = ssh_pubkey.unwrap();
     let priv_pubkey = privkey.pubkey.clone();
-    let signer:SigningFunction = privkey.into();
 
     let user_cert = Certificate::builder(&ssh_pubkey, CertType::User, &priv_pubkey).unwrap()
         .serial(0xFEFEFEFEFEFEFEFE)
@@ -668,9 +658,8 @@ fn create_sign_parse_verify_rsa2048_impl_into() {
         .principal("obelisk")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
-        .set_extensions(Extensions::Standard)
-        .sign(signer);
+        .set_extensions(Certificate::standard_extensions())
+        .sign(&privkey);
 
     match &user_cert {
         Ok(_) => (),
@@ -726,11 +715,10 @@ fn create_sign_parse_verify_ed25519ca_chained_method_invocation() {
         .principal("mitchell")
         .valid_after(0)
         .valid_before(0xFFFFFFFFFFFFFFFF)
-        .set_critical_options(CriticalOptions::None)
         .critical_option("test", "test_value")
-        .set_extensions(Extensions::Standard)
+        .set_extensions(Certificate::standard_extensions())
         .extension("extension_test", "extension_test_value")
-        .sign(create_signer(privkey));
+        .sign(&privkey);
 
     assert!(user_cert.is_ok());
 
